package com.lionel.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;

public class JdbcDao {
	private static JdbcDao instance = null;

	private JdbcDao(String serverUrl) {
		try {
			Class.forName(MySqlConfig.MYSQL_DRIVER);
			conn = DriverManager.getConnection(serverUrl,
					MySqlConfig.MYSQL_USERNAME, MySqlConfig.MYSQL_PASSWORD);
		} catch (Exception e) {
			System.out.println("Mysql: Cannot create connection");
			e.printStackTrace();
		}
	}

	public static JdbcDao getInstance(int serverNumber, String databaseName) {
		String serverUrl = String.format(MySqlConfig.MYSQL_URL, serverNumber,
				databaseName);
		if (instance == null) {
			instance = new JdbcDao(serverUrl);
		}
		return instance;
	}

	private Connection conn;
	private PreparedStatement pstmt;

	public int executeUpdate(String sql, Object... params) {
		try {
			pstmt = conn.prepareStatement(sql);
			for (int i = 0; i < params.length; ++i) {
				pstmt.setObject(i + 1, params[i]);
			}

			int res = pstmt.executeUpdate();
			pstmt.close();
			return res;
		} catch (SQLException e) {
			System.out.println(e.getMessage());
		} finally {
			try {
				pstmt.close();
			} catch (SQLException e) {
				System.out.println(e.getMessage());
			}
		}
		return -1;
	}
}
