package com.lionel.linq;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.lionel.linq.expressions.Expression;
import com.lionel.linq.expressions.Func;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.ParseException;

/**
 * LINQ-to-Objects is a set of extension methods on Enumerable that allow you to
 * perform in-memory query operations on arbitrary sequences of objects. The
 * methods accept simple delegates when necessary.
 * 
 * @author huynt3
 * 
 */
public class Enumerable {
	/**
	 * Filters a sequence of values based on a predicate.
	 * 
	 * @param source
	 * @param predicate
	 * @return
	 */
	public static <S> Iterable<S> where(Iterable<S> source,
			Func<S, Boolean> predicate) {
		Iterator<S> iterator = source.iterator();
		S next = null;
		List<S> result = new ArrayList<S>();
		while (iterator.hasNext()) {
			next = iterator.next();
			if (predicate.invoke(next)) {
				result.add(next);
			}

		}
		return result;
	}

	/**
	 * Filters a sequence of values based on an expression. Expression is a
	 * string with pattern argument -> function of argument
	 * 
	 * @param source
	 * @param expression
	 * @return
	 * @throws ParseException
	 */
	@Deprecated
	public static <S> Iterable<S> where(Iterable<S> source, String expression) {
		String[] exprArray = expression.split(" ");
		if (exprArray.length < 5) {
			return null;
		}
		Func<String[], Integer> predicate = new Func<String[], Integer>() {
			@Override
			public Integer invoke(String[] args) {
				for (int i = 0; i < args.length; ++i) {
					if (args[i].equals("->")) {
						return i;
					}
				}
				return -1;
			}
		};
		int lambdaOperatorIndex = predicate.invoke(exprArray);
		if (lambdaOperatorIndex == -1) {
			return null;
		}
		Type type = source.iterator().next().getClass();
		System.out.println(type);
		for (int i = 0; i < lambdaOperatorIndex; ++i) {
			Expression.parameter(type, exprArray[i]);
		}
		return null;
	}
}
