package com.lionel.linq.expressions;

import java.lang.reflect.Type;

public class BinaryExpression extends Expression {
	private Expression left;
	private Expression right;
	private ExpressionType binaryType;

	public BinaryExpression(Expression left, ExpressionType binaryType,
			Expression right) {
		this.left = left;
		this.binaryType = binaryType;
		this.right = right;
	}

	public Expression getLeft() {
		return left;
	}

	public Expression getRight() {
		return right;
	}

	public ExpressionType getBinaryType() {
		return binaryType;
	}

	public boolean canReduce() {
		if (left == null || right == null) {
			return false;
		} else {
			Type leftType = ((Expression) left).getType();
			Type rightType = ((Expression) right).getType();
			return leftType == rightType
					|| (leftType == Integer.class && rightType == Double.class)
					|| (rightType == Integer.class && leftType == Double.class);
		}
	}

	public Object reduce() {
		if (canReduce()) {
			Object leftValue = ((ConstantExpression) left).getValue();
			Object rightValue = ((ConstantExpression) right).getValue();
			Type leftType = ((Expression) left).getType();
			Type rightType = ((Expression) right).getType();

			switch (binaryType) {
			case Add:
				if (leftType == Integer.class || leftType == Double.class) {
					if (leftType == Integer.class && rightType == Integer.class) {
						return add(((Number) leftValue).intValue(),
								((Number) rightValue).intValue());
					} else {
						return add(((Number) leftValue).doubleValue(),
								((Number) rightValue).doubleValue());
					}
				} else if (leftType == Boolean.class) {
					return (Boolean) leftValue && (Boolean) rightValue;
				} else {
					return leftValue.toString() + rightValue.toString();
				}
			case Divide:
				break;
			case Equal:
				break;
			case GreaterThan:
				break;
			case GreaterThanOrEqual:
				break;
			case LessThan:
				if (leftType != Integer.class && leftType != Double.class) {
					return false;
				} else {
					return new Double(leftValue.toString()) < new Double(rightValue.toString());
				}
			case LessThanOrEqual:
				break;
			case Multiply:
				break;
			case Subtract:
				break;
			default:
				break;
			}
		}
		return null;
	}

	public int add(int left, int right) {
		return left + right;
	}

	public double add(double left, double right) {
		return left + right;
	}
}
