package com.lionel.linq.expressions;

import java.lang.reflect.Type;

public abstract class Expression {
	protected Type type;
	
	public Type getType() {
		return type;
	}
	
	public static ParameterExpression parameter(Type type, String name) {
		return new ParameterExpression(type, name);
	}
	
	public static LambdaExpression lamda(Expression method) {
		return new LambdaExpression(method);
	}
	
	public static LambdaExpression lamda(Expression method, ParameterExpression paramExprs) {
		return new LambdaExpression(method, paramExprs);
	}
	
	public static LambdaExpression lamda(Func<?,?> func, Expression method, ParameterExpression paramExprs) {
		return new LambdaExpression(func, method, paramExprs);
	}
	
	public static BinaryExpression makeBinary(ExpressionType binaryType, Expression left, Expression right) {
		return new BinaryExpression(left, binaryType, right);
	}
	
	public static BinaryExpression add(Expression left, Expression right) {
		return makeBinary(ExpressionType.Add, left, right);
	}
	
	public static BinaryExpression lessThan(Expression left, Expression right) {
		return makeBinary(ExpressionType.LessThan, left, right);
	}
	
	public static ConstantExpression constant(Object value) {
		return new ConstantExpression(value);
	}
	
	public static ConstantExpression constant(Object value, Type type) {
		return new ConstantExpression(value, type);
	}
}
