package com.lionel.linq.expressions;

import java.lang.reflect.Type;

public class LambdaExpression extends Expression {
	private ParameterExpression parameters;
	private ExpressionType nodeType;
	private Type returnType;
	private BinaryExpression body;

	public ParameterExpression getParameters() {
		return parameters;
	}

	public ExpressionType getNodeType() {
		return nodeType;
	}

	private String getBinaryType() {
		switch (nodeType) {
		case Add:
			return "+";
		case Divide:
			return "/";
		case Equal:
			return "=";
		case GreaterThan:
			return ">";
		case GreaterThanOrEqual:
			return ">=";
		case LessThan:
			return "<";
		case LessThanOrEqual:
			return "<=";
		case Multiply:
			return "*";
		case Subtract:
			return "-";
		default:
			return "";
		}
	}

	public Type getReturnType() {
		return returnType;
	}

	public BinaryExpression getBody() {
		return body;
	}

	public LambdaExpression(Expression method) {
		this.body = (BinaryExpression) method;
		this.parameters = (ParameterExpression) this.body.getLeft();
		this.nodeType = this.body.getBinaryType();
	}

	public LambdaExpression(Expression method, ParameterExpression paramExprs) {
		this.parameters = paramExprs;
		this.body = (BinaryExpression) method;
	}

	public LambdaExpression(Func<?, ?> func, Expression method,
			ParameterExpression paramExprs) {
		this.parameters = paramExprs;
		this.body = (BinaryExpression) method;
	}

	public Func<?, ?> compile() {
		final Object rightValue = ((ConstantExpression) this.body.getRight())
				.getValue();
		Func<?, ?> func = null;
		switch (this.nodeType) {
		case Add:
			func = new Func<Object, Object>() {
				@Override
				public Object invoke(Object leftValue) {
					if (leftValue.getClass() == Integer.class
							|| leftValue.getClass() == Double.class) {
						if (leftValue.getClass() == Integer.class
								|| rightValue.getClass() == Integer.class) {
							return ((Number) leftValue).intValue()
									+ ((Number) rightValue).intValue();
						} else {
							return ((Number) leftValue).doubleValue()
									+ ((Number) rightValue).doubleValue();
						}
					} else {
						return leftValue.toString() + rightValue.toString();
					}
				}
			};
			break;
		case Divide:
			return null;
		case Equal:
			return null;
		case GreaterThan:
			return null;
		case GreaterThanOrEqual:
			return null;
		case LessThan:
			func = new Func<Object, Boolean>() {
				@Override
				public Boolean invoke(Object leftValue) {
					return new Double(leftValue.toString()) < new Double(
							rightValue.toString());
				}
			};
			break;
		case LessThanOrEqual:
			return null;
		case Multiply:
			return null;
		case Subtract:
			return null;
		default:
			break;
		}
		return func;
	}

	public String toString() {
		String expr = "{argument} -> ({argument} {operator} {constant})";
		expr = expr.replace("{argument}",
				((ParameterExpression) this.body.getLeft()).getName());
		String binaryType = this.getBinaryType();
		expr = expr.replace("{operator}", binaryType);
		expr = expr.replace("{constant}",
				((ConstantExpression) this.body.getRight()).toString());
		return expr;
	}

}
